using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Threading;
using System.IO;

namespace PlayersRunner
{
    public partial class FormMain : Form
    {
        private static string GENERAL = "general";
        private static string PLAYERS = "players";
        private static string PLAYER = "p";

        private static string TEAMNAME = "team";
        private static string HOSTNAME = "host";
        private static string PEXECUTABLE = "pexe";
        private static string GEXECUTABLE = "gexe";
        private static string CEXECUTABLE = "cexe";

        private static string TEAMATTR = "%TEAM%";
        private static string HOSTATTR = "%HOST%";

        private static string SERVERPROCESS = "rcssserver.exe";
        private static string SERVERPROCESSDIR = "server";
        private static string WORKINGDIR = "wd";
        private static string GOALIE_FIRST = "goalie_first";

        Process pcsServer = null;

        private List<Process> players = new List<Process>();
        private List<Process> opponents = new List<Process>();

        public FormMain()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Creates player item name in INI file
        /// </summary>
        /// <param name="nr">number between 1..11</param>
        /// <returns>player item name in INI file</returns>
        private string GetPlayerName(int nr)
        {
            return PLAYER + nr.ToString();
        }

        /// <summary>
        /// Creates attributes from parameterized attributes
        /// </summary>
        /// <param name="attr"></param>
        /// <param name="team"></param>
        /// <param name="host"></param>
        /// <returns></returns>
        private string GetAttributes(string attr, string team, string host)
        {
            return attr.Replace(TEAMATTR, team).Replace(HOSTATTR, host);
        }

        /// <summary>
        /// Adds player to players list and waits for execution
        /// </summary>
        /// <param name="nr"></param>
        /// <param name="team"></param>
        /// <param name="host"></param>
        /// <param name="exe"></param>
        /// <param name="wd"></param>
        /// <param name="list"></param>
        /// <returns></returns>
        private void AddPlayer(int nr, string team, string host, string exe, IniFile file, string wd, List<Process> list)
        {
            string attr = GetAttributes(
                file.IniReadValue(PLAYERS, GetPlayerName(nr)),
                team,
                host);

            if (!string.IsNullOrEmpty(attr))
            {
                Process p = CreateProcess(exe, attr, wd);
                list.Add(p);
                Thread.Sleep(50);
            }
        }

        private void LoadConfig(string filename, List<Process> list)
        {
            if (list != null)
                list.Clear();

            IniFile file = new IniFile(filename);
            // general section
            string team = file.IniReadValue(GENERAL, TEAMNAME);
            string host = file.IniReadValue(GENERAL, HOSTNAME);
            string pexe = file.IniReadValue(GENERAL, PEXECUTABLE);
            string gexe = file.IniReadValue(GENERAL, GEXECUTABLE);
            string cexe = file.IniReadValue(GENERAL, CEXECUTABLE);
            string wd = file.IniReadValue(GENERAL, WORKINGDIR);
            bool gf = file.IniReadValue(GENERAL, GOALIE_FIRST).Trim().Equals("1");

            // goalie first?
            if (gf)
            {
                AddPlayer(11, team, host, gexe, file, wd, list);
                Thread.Sleep(300);
            }

            // players section
            for (int i = 1; i <= 12; i++)
            {
                switch (i)
                {
                    case 11: //goalie
                        if (!gf)
                            AddPlayer(i, team, host, gexe, file, wd, list);
                        break;
                    case 12: // coach
                        AddPlayer(i, team, host, cexe, file, wd, list);
                        break;
                    default: // player
                        AddPlayer(i, team, host, pexe, file, wd, list);
                        break;
                }
            }
        }

        /// <summary>
        /// Creates process to run in backgroud
        /// </summary>
        /// <param name="filename"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        private Process CreateProcess(string filename, string args)
        {
            return CreateProcess(filename, args, Application.StartupPath);
        }

        /// <summary>
        /// Creates process to run in backgroud
        /// </summary>
        /// <param name="filename"></param>
        /// <param name="args"></param>
        /// <param name="dir"
        /// <returns></returns>
        private Process CreateProcess(string filename, string args, string dir)
        {
            ProcessStartInfo info = new ProcessStartInfo();
            info.CreateNoWindow = true;
            info.ErrorDialog = true;
            if (File.Exists(filename))
            {
                if (!string.IsNullOrEmpty(dir))
                    info.WorkingDirectory = dir;
                else 
                    info.WorkingDirectory = Directory.GetParent(filename).FullName;
                info.FileName = filename;
            }
            else if (File.Exists(dir + "\\" + filename))
            {
                if (!string.IsNullOrEmpty(dir))
                    info.WorkingDirectory = dir;
                else
                    info.WorkingDirectory = dir;
                info.FileName = dir + "\\" + filename;
            }
            info.Arguments = args;
            info.RedirectStandardOutput = false;
            info.RedirectStandardInput = false;
            info.WindowStyle = ProcessWindowStyle.Hidden;
            //info.UseShellExecute = false;

            Process p = Process.Start(info);
            p.EnableRaisingEvents = true;

            return p;
        }

        private void Kill()
        {
            Process p = pcsServer;
            if (p == null)
                p = ServerProcess();

            if (p != null)
            {
                if (!p.HasExited)
                {
                    p.Kill();
                    p.WaitForExit();
                }
                pcsServer = null;
            }
            Thread.Sleep(3000);
            KillConfig(players);
            KillConfig(opponents);
        }

        private void KillConfig(List<Process> list)
        {
            foreach (Process p in list)
            {
                if (!p.HasExited)
                {
                    p.Kill();
                    p.WaitForExit();
                }
            }
        }

        private Process ServerProcess()
        {
            Process[] p = Process.GetProcessesByName(SERVERPROCESS);
            if (p.Length > 0)
                return p[0];
            return null;
        }

        private bool IsServerRunning()
        {
            return ServerProcess() != null;
        }

        private void RunServer()
        {
            pcsServer = CreateProcess(SERVERPROCESS, "", Application.StartupPath + "\\" + SERVERPROCESSDIR);
            Thread.Sleep(300);
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            if (!IsServerRunning())
                RunServer();
            else
            {
                if (MessageBox.Show("Server is already running!\nTerminate?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    ServerProcess().Kill();
                    RunServer();
                }
            }
            LoadConfig(txbTeammates.Text, players);
            LoadConfig(txbOpponents.Text, opponents);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            Kill();
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (MessageBox.Show("Really terminate game?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                Kill();
            else
                e.Cancel = true;
        }
    }
}